/**
 * @file ADC.c
 * A Simple driver to handle ADC-Related functionality
 *
 *  ------------------------------------
 *  @author	Andrew Retallack, Crash-Bang Prototyping
 *			www.crash-bang.com
 *  @date	20/06/2015
 */ 


#include "ADC.h"

/***********************************************************************
*
* This routine initialises the ADC by:
*   1. setting input channel
*   2. setting prescaler
*   3. enabling ADC
*
* INPUT:	adcChannel The ADC Channel to read from
* RETURNS:	none
************************************************************************/
void ADC_init(uint8_t adcChannel)
{

	//Reference Voltage = AREF Pin
	//ADMUX &= ~( (1<<REFS0) | (1<<REFS1) );	
	
	//Reference Voltage = VCC
	ADMUX &= ~( (1<<REFS1) );	
	ADMUX |= (1<<REFS0);	
	
	ADMUX |= adcChannel;					//The ADC Channel: 0 = ADC0 - 5 = ADC5

	ADCSRA = (1<<ADPS2) | (1<<ADPS1) | (1<<ADPS0);	//Clock Prescaler: 128, allow for accurate (>50kHz <200kHz) sampling with 16MHz system clock

	ADCSRA |= (1<<ADEN);			//Enable the ADC
	
}


/***********************************************************************
*
* This routine takes a reading from the initialised ADC channel
*
* INPUT:	none
* RETURNS:	the value of the reading (0 - 1023)
************************************************************************/
uint16_t ADC_read(void)
{

	uint16_t adcValue;
	
	ADCSRA |= (1<<ADSC);		//Write a "1" to start the conversion.  Bit clears when conversion is complete.
	
	while (ADCSRA & (1<<ADSC));	//Wait for the conversion to finish - ADSC bit goes low on completion
	
	adcValue = ADC;
	
	return adcValue;		//Return the ADC value from 0 to 1023

}
